import collections
import contextlib
from aiortc import MediaStreamTrack
from aiortc.contrib.media import MediaBlackhole, MediaPlayer, MediaRecorder, MediaRelay
import webrtcvad
import wave

class AudioTrackVad(MediaStreamTrack):
	def __init__(self, track, stage=3, onvoiceend=None):
		super().__init__()
		self.track = track
		self.onvoiceend = onvoiceend
		self.vad = webrtcvad.Vad(stage)
		self.sample_rate = self.track.getSettings().sampleRate
		frameSize = self.track.getSettings().frameSize
		self.frame_duration_ms = (1000 * frameSize) / self.sample_rate
		self.num_padding_frames = 10
		self.ring_buffer = collections.deque(maxlen=self.num_padding_frames)
		self.triggered = False
		self.voiced_frames = []
	
	async def recv(self):
		frame = await self.track.recv()
		self.vad_check(frame)
		return frame
	
	async def vad_check(self, frame):
		is_speech = self.vad.is_speech(frame, self.sample_rate)
		if not self.triggered:
			self.ring_buffer.append((frame, is_speech))
			num_voiced = len([f for f, speech in self.ring_buffer if speech])
			# If we're NOTTRIGGERED and more than 90% of the frames in
			# the ring buffer are voiced frames, then enter the
			# TRIGGERED state.
			if num_voiced > 0.9 * self.ring_buffer.maxlen:
				self.triggered = True
				# We want to yield all the audio we see from now until
				# we are NOTTRIGGERED, but we have to start with the
				# audio that's already in the ring buffer.
				for f, s in self.ring_buffer:
					self.voiced_frames.append(f)
				self.ring_buffer.clear()
		else:
			# We're in the TRIGGERED state, so collect the audio data
			# and add it to the ring buffer.
			self.voiced_frames.append(frame)
			self.ring_buffer.append((frame, is_speech))
			num_unvoiced = len([f for f, speech in self.ring_buffer if not speech])
			# If more than 90% of the frames in the ring buffer are
			# unvoiced, then enter NOTTRIGGERED and yield whatever
			# audio we've collected.
			if num_unvoiced > 0.9 * self.ring_buffer.maxlen:
				self.triggered = False
				audio_data =  b''.join([f.bytes for f in voiced_frames])
				self.write_wave(audio_data)
				self.ring_buffer.clear()
				voiced_frames = []

	async def write_wave(self, audio_data):
		"""Writes a .wav file.

		Takes path, PCM audio data, and sample rate.
		"""
		path = make_temp(subfix='.wav')
		with contextlib.closing(wave.open(path, 'wb')) as wf:
			wf.setnchannels(1)
			wf.setsampwidth(2)
			wf.setframerate(self.sample_rate)
			wf.writeframes(audio)
		
		if self.onvoiceend:
			self.onvoiceend(path)

